extends CharacterBody3D


const SPEED = 5.0
const JUMP_VELOCITY = 4.5
const MOUSE_SENSITIVITY = 700

@onready var camera = $Head/Camera
var rotation_target := Vector3.ZERO
var mouse_captured := false

func _ready():
	Input.mouse_mode = Input.MOUSE_MODE_CAPTURED
	mouse_captured = true

func _physics_process(delta: float) -> void:
	# Rotation
	rotation_target.y = clamp(rotation_target.y, deg_to_rad(-90), deg_to_rad(90))
	rotation.y = lerp_angle(rotation.y, rotation_target.x, delta * 25)
	camera.rotation.x = lerp_angle(camera.rotation.x, rotation_target.y, delta * 25)


	# Add the gravity.
	if not is_on_floor():
		velocity += get_gravity() * delta

	# Handle jump.
	if Input.is_action_just_pressed("jump") and is_on_floor():
		velocity.y = JUMP_VELOCITY

	# Get the input direction and handle the movement/deceleration.
	# As good practice, you should replace UI actions with custom gameplay actions.
	var input_dir := Input.get_vector("move_left", "move_right", "move_forward", "move_back")
	var direction := (transform.basis * Vector3(input_dir.x, 0, input_dir.y)).normalized()
	if direction:
		velocity.x = direction.x * SPEED
		velocity.z = direction.z * SPEED
	else:
		velocity.x = move_toward(velocity.x, 0, SPEED)
		velocity.z = move_toward(velocity.z, 0, SPEED)

	move_and_slide()

func _input(event: InputEvent) -> void:
	if event is InputEventMouseMotion and mouse_captured:
		rotation_target.y -= event.relative.y / MOUSE_SENSITIVITY
		rotation_target.x -= event.relative.x / MOUSE_SENSITIVITY

func _process(delta: float) -> void:
	if Input.is_action_just_pressed("mouse_capture"):
		Input.mouse_mode = Input.MOUSE_MODE_CAPTURED
		mouse_captured = true
	
	if Input.is_action_just_pressed("mouse_capture_exit"):
		Input.mouse_mode = Input.MOUSE_MODE_VISIBLE
		mouse_captured = false
